#
# Copyright (c) 2004-2011 Purdue University All rights reserved.
# 
# Developed by: HUBzero Technology Group, Purdue University
#               http://hubzero.org
# 
# HUBzero is free software: you can redistribute it and/or modify it under the terms of the
# GNU Lesser General Public License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
# 
# HUBzero is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.  You should have received a
# copy of the GNU Lesser General Public License along with HUBzero.
# If not, see <http://www.gnu.org/licenses/>.
# 
# GNU LESSER GENERAL PUBLIC LICENSE
# Version 3, 29 June 2007
# Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
#
import os.path
import re

class LocalBatchAppScript:
   def __init__(self,
                localJobId,
                instanceId,
                executable,
                arguments,
                useEnvironment,
                environment,
                timeHistoryLogs):
      self.localJobId           = localJobId
      self.instanceId           = instanceId
      self.executable           = executable
      self.arguments            = arguments
      self.useEnvironment       = useEnvironment
      self.environment          = environment
      self.isMultiCoreRequest   = False
      self.timestampTransferred = timeHistoryLogs['timestampTransferred']
      self.timestampStart       = timeHistoryLogs['timestampStart']
      self.timestampFinish      = timeHistoryLogs['timestampFinish']
      self.timeResults          = timeHistoryLogs['timeResults']
      if 'exitCode' in timeHistoryLogs:
         self.exitCode          = timeHistoryLogs['exitCode']
      else:
         self.exitCode          = ""


   def __makeSerialTemplate(self):
      return """#!/bin/sh
# LocalBatchAppScript:makeSerialTemplate
#
trap cleanup HUP INT QUIT ABRT TERM

cleanup()
{
   echo "Abnormal termination by signal"
   if [ ! -s TS_FINISH ] ; then
      date +"%s" > TS_FINISH
   fi
   exit 1
}

exitCode=0

cd INSTANCEDIRECTORY
export PATH=INSTANCEDIRECTORY:${PATH}

USEENVIRONMENT
ENVIRONMENT

TIMEPATH=
for timePath in ${HOME}/bin/time /usr/bin/time /usr/local/bin/time /apps/submit/bin/time ; do
   if [ -x ${timePath} ] ; then
      TIMEPATH=${timePath}
      break
   fi
done

date +"%s" > TS_START

${TIMEPATH} -p -o TIME_RESULTS EXECUTABLE ARGUMENTS > JOBID.stdout 2> JOBID.stderr
exitCode=$?

date +"%s" > TS_FINISH

EXITCODE

exit ${exitCode}
"""


   def __buildAppSerialScript(self):
      # setup regex's for the template
      re_jobid             = re.compile("JOBID")
      re_instanceDirectory = re.compile("INSTANCEDIRECTORY")
      re_exe               = re.compile("EXECUTABLE")
      re_arguments         = re.compile("ARGUMENTS")
      re_useEnvironment    = re.compile("USEENVIRONMENT")
      re_environment       = re.compile("ENVIRONMENT")
      re_tsTransferred     = re.compile("TS_TRANSFERRED")
      re_tsStart           = re.compile("TS_START")
      re_tsFinish          = re.compile("TS_FINISH")
      re_timeResults       = re.compile("TIME_RESULTS")
      re_exitCode          = re.compile("EXITCODE")

      template = self.__makeSerialTemplate()

      template = re_jobid.sub("%s_%s" % (self.localJobId,self.instanceId),template)
      template = re_instanceDirectory.sub(os.path.join(os.getcwd(),self.localJobId,self.instanceId),template)
      template = re_exe.sub(self.executable,template)
      template = re_arguments.sub(self.arguments,template)
      template = re_useEnvironment.sub(self.useEnvironment,template)

      environmentExport = ""
      environmentVars = self.environment.split()
      for environmentVar in environmentVars:
         environmentExport += "export " + environmentVar + "\n"
      template = re_environment.sub(environmentExport,template)

      template = re_tsTransferred.sub(self.timestampTransferred,template)
      template = re_tsStart.sub(self.timestampStart,template)
      template = re_tsFinish.sub(self.timestampFinish,template)
      template = re_timeResults.sub(self.timeResults,template)
      if self.exitCode:
         template = re_exitCode.sub("echo ${exitCode} > %s\nexitCode=0" % (self.exitCode),template)
      else:
         template = re_exitCode.sub("",template)
   
      return(template)


   def buildAppScript(self):
      appScriptName = "%s_%s.sh" % (self.localJobId,self.instanceId)
      if self.isMultiCoreRequest:
         appScript = ""
      else:
         appScript = self.__buildAppSerialScript()

      return(appScriptName,appScript)


